% Use smoothing to compute the polar oracle for group overlapping 
%  [v, supp, fA, opt_obj] = polar_op_group(g, opt)
% Input:
%   g: weights on the nodes
%   par: parameters. Eg, par.graph.G_by_V gives the graph.
% Output:
%  v = pola(g) with 0's in the complement of C(g) ignored
%  supp = C(g), a 0/1 vector representing the optimal set A
%  fA = F(C(g))^{1/q}
%  opt_obj: the optimal value of <g.^q, 1_A> / F(A)

function [v, supp, fA, opt_obj] = polar_op_smooth_group(g, par)
  
  exp_par = par.exp_par;
  sbd_ctx = par.sbd_ctx;
  
  [num_var, num_group] = size(sbd_ctx.V_by_G);
  p = exp_par.p;
  if isinf(p), q = 1; else q = p/(p-1); end    
  s = abs(g).^q;
  
  global y_g
  
  obj = @(y)group_PO_smooth_obj(y, s, par.polar_par.sigma, sbd_ctx.G_by_V);

  % Solve the smooth objective
%   y_g = ones(num_group, 1)/num_group;
  [y_g, f] = minConf_PQN(obj, y_g, @projectSimplex, par.polar_par.minConf_PQN_opt);
    
  % Now recover the solution from y
  [~, I] = sort(y_g);
  % Support is on nodes, num_act_group is on groups
  [supp, num_act_group, obj1] = group_PO_recover(s, sbd_ctx.V_by_G(:, I));
  fA = num_act_group;

  % Compute the dual vector
  v = g(supp);
  sum_sA = sum(s(supp));
  if isinf(p)
    vv = sign(v);
  else
    vv = sign(v) .* (abs(v).^(q-1)) / sum_sA^(1/p);
  end
  % Check by Holder's inequality
  assert(abs(v'*vv - sum_sA^(1/q)) < 1e-6);
  
  if par.polar_par.verbose > 1
    fprintf('minConf = %g, recvy = %g\n', f, sum_sA/num_act_group);
  end
  
  opt_obj = sum_sA / fA;
  fA = fA^(1/q);
  v = vv / fA;
  
%   assert(abs(f + obj1) < 1e-5);
  
  % Double check by solving LP
  if par.polar_par.check_LP
    graph = par.graph;
    ntotal = num_group + num_var;
    [sol, f] = linprog([-s; zeros(num_group,1)], graph.A, graph.b, ...
                graph.Aeq, graph.beq, zeros(ntotal,1), ones(ntotal,1), ...
                [zeros(num_var,1); ones(num_group,1)/num_group]);
    cprintf('red', 'LP obj = %g\n', f);
  end
  
end
