function [output cache] = functions_obj_polar(objf, fname, model, searchDir, data, params, cache)
% Set of functions that are specific to matrix completion problem
%
% Total space = St(n,p)\times Cone(p)\times St(m,p)
% St(n,p): Stiefel manifold
% Cone(p): Positive definite matrices of size p\times p
%
% Usages:
%
% Cost function:                            f
% Gradient on the total space:              grad 
% Riemmanian Hessian on the total space:    hess 
% Dual variable:                            dual_var
% model is the current iterate
% eta is a search direction
% params is a cell array that contains {I,J,trueDists,EIJ} (see lowrank_dist_completion.m)
% cache is a cell array to reduce redundant computations
%
% Authors:
% Bamdev Mishra and Gilles Meyer
% {b.mishra,g.meyer}@ulg.ac.be

switch lower(fname),   
        
  case 'f', % Objective function      
    output = objf({model.U*model.B, model.V}) + model.lambda*trace(model.B);      

  case 'grad_f', % Compute gradient      
    [~, S] = objf({model.U*model.B, model.V});

    grad.U = cell_times_mat(S, model.V) *model.B;
    grad.U = proj_tangent_space_stiefel(grad.U,model.U);

    grad.B = model.B*(symm(model.U'*cell_times_mat(S, model.V))) * model.B ...
              + model.lambda*model.B*model.B;

    grad.V = cell_trans_times_mat(S, model.U) * model.B;
    grad.V = proj_tangent_space_stiefel(grad.V,model.V);

    % Cache the value of S and gradient 
    % for subsequent computation of the Hessian
    cache.S = S;
    cache.grad = grad;

    output = grad;

  case 'hessian', % Compute hessian

    if isfield(cache,'S'),
      S = cache.S;
    else
      [~, S] = objf({model.U*model.B, model.V});
    end

    if isfield(cache, 'grad'),
      grad = cache.grad;
    else
      grad = functions_obj_polar(objf, 'grad_f', model, [], data);
    end

    [~, ~, S_star] = objf({model, searchDir}, 'dirHes');

    hessian.U = cell_times_mat(S, model.V*searchDir.B + searchDir.V*model.B) ...
                  + cell_times_mat(S_star, model.V)*model.B;

    % Projection onto the tangent space to the Stiefel Manifold      
    hessian.U = proj_tangent_space_stiefel(hessian.U, model.U) ...
                - proj_tangent_space_stiefel(searchDir.U ...
                * symm(model.U'*(cell_times_mat(S, model.V)*model.B)),model.U);

    % Compute directional derivative      
    symmUSV = symm(model.U'*cell_times_mat(S, model.V));

    hessian.B = searchDir.B*symmUSV*model.B  ...
                + model.B*symmUSV*searchDir.B  ...
                + model.B*symm(searchDir.U'*cell_times_mat(S, model.V) ...
                  + model.U'*(cell_times_mat(S_star, model.V) ...
                    + cell_times_mat(S, searchDir.V)))*model.B ...
                + model.lambda*model.B*searchDir.B ...
                + model.lambda*searchDir.B*model.B;

    % Correction for non-constant metric
    hessian.B = hessian.B - symm(searchDir.B*(model.B\grad.B));

    hessian.V = cell_trans_times_mat(S, model.U*searchDir.B + searchDir.U*model.B) ...
                + cell_trans_times_mat(S_star, model.U*model.B);

    % Projection onto the tangent space to the Stiefel Manifold
    hessian.V = proj_tangent_space_stiefel(hessian.V,model.V) - ...
                proj_tangent_space_stiefel(searchDir.V ...
                *symm(model.V'*cell_trans_times_mat(S, model.U)*model.B),model.V);

    output = hessian;

  otherwise,
    error('Unknown function %s', lower(fname));
end
