function [w, f, res] = gcg_vec(loss, po, reg, w0, par)
% Inputs:
%   loss: the loss function handle which can be called by [f, g] = loss(x);
%   po: the polar oracle: 
%     Prototype:
%       [v, supp, fA] = pola(g);
%       Input:
%         G: the gradient to invoke the polar oracle
%         opt: options passed to the polar oracle
%              opt.weight_reg: the weight used in the prox operator
%       Output:
%         v: pola(g) with 0's in the complement of C(g) ignored
%         supp: reg(Z), value of regularizer at Z (not multiplied by weight_reg)
%         fA: F(C(g))^{1/q}
%   reg: the function handle that computes the regularization value
%   w0: initial solution
%   par: parameters
% 
% Outputs:
%   w: the optimal solution
%   f: the optimal objective value
  
  n = length(w0);
  w = zeros(n, 1);
  fA = [];
  num_atom = 0;
  x = [];   % u^i is represented as a vector to allow direct call of lbfgsb
  s = 0;
  exp_par = par.exp_par;
  cg_par = par.cg_par;
  cone_par = par.cone_par;
  local_par = par.local_par;
  polar_par = par.polar_par;
  
  lambda = exp_par.lambda;
  max_iter = cg_par.max_cg_iter;
  
  % use x, supp_list, and fA to record the bases
  supp_list = cell(max_iter, 1);
  fA = zeros(max_iter, 1);
  s_list = zeros(max_iter, 1);
    
  zero_const = zeros(max_iter,1);
  res = struct('flist', zero_const, ...
               'flist_ori', zero_const);
  rtime = struct('tm_all', zero_const, ...
                 'tm_loss', zero_const, ...
                 'tm_po', zero_const, ...
                 'tm_conic', zero_const, ...
                 'tm_local', zero_const);
  
  threshold = 1e-5;
  
  for i = 1 : max_iter-1
    
    reg_val = reg(w);
    
    tstart = cputime;
    [f, g] = loss(w);
    rtime.tm_loss(i) = cputime - tstart;
    
    f_ori = f + lambda*reg_val;
    res.flist_ori(i) = f_ori;

    num_atom = num_atom + 1;
    t1 = cputime;
    [v, supp_list{num_atom}, fA(num_atom), obj] = po(-g, par); % v is compact with no zero
    if isfield(par, 'backup_pola') %&& i == 1
      [vv, ss, ffA, dd] = par.backup_pola(-g, par); % v is compact with no zero
      cprintf('red', 'obj_sel = %g, obj_backup = %g\n', obj, dd);
      if i == 1
        v = vv; supp_list{num_atom} = ss; fA(num_atom) = ffA;
      end
    end
    
    rtime.tm_po(i) = cputime - t1;
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Conic search
    t1 = cputime;
    switch cg_par.conic_search
      case '2d'        
        [ab, f, iter, feval, flag] = lbfgsb([1;0], [0;0], [inf;inf], ...
                                            @obj_2d_cone, [], ...
                                            cone_par.monitor, cone_par);        
        a = ab(1);  b = ab(2);
        x = [a*x; b*v];
        if cone_par.print_summary
          cprintf('red', '%d: cone: f=%f, iter=%d, feval=%d, a=%f, b=%f, msg=%s\n', ...
                  i, f, iter, feval, a, b, msg2str(flag, 'lbfgsb'));
        end

      case 'total'
        [wgts, f, iter, feval, flag] = lbfgsb([ones(num_atom-1,1);0], ...
                                            zeros(num_atom, 1), Inf(num_atom, 1), ...
                                            @obj_total_cone, [], ...
                                            cone_par.monitor, cone_par);
        idx = 1;
        for j = 1 : num_atom-1
          newidx = idx + length(supp_list{j}) - 1;
          x(idx:newidx) = wgts(j)*x(idx:newidx);
          idx = newidx + 1;
          s_list(j) = s_list(j)*wgts(j);
        end
        x = [x; wgts(num_atom)*v];
        s_list(num_atom) = wgts(num_atom);
        
        if cone_par.print_summary
              cprintf('red', '%d: cone: f=%f, iter=%d, feval=%d, msg=%s\n', ...
                      i, f, iter, feval, msg2str(flag, 'lbfgsb'));
        end

      case 'const'
        % use 1/k fixed updates
        b = 1/(1+i);  a = 1 - b;
        x = [a*x; b*v];
    end       
    rtime.tm_conic(i) = cputime - t1;

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % local optimization
    if cg_par.use_local
      bu = Inf(length(x), 1);
      t1 = cputime;
      [x, f, iter, feval, flag] = lbfgsb(x, -bu, bu, @obj_local, [], ...
                                        local_par.monitor, local_par);
      rtime.tm_local(i) = cputime - t1;
%       if local_par.print_summary
        cprintf('red', '%d: local: f=%f, iter=%d, feval=%d, msg=%s\n', ...
                i, f, iter, feval, msg2str(flag, 'lbfgsb'));    
%       end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    w = xlist_2_w(x);
    rtime.tm_all(i) = cputime - tstart;
    
    % Update sum of weights
    res.flist(i) = f;
    s = 0;  idx = 1;
    for k = 1:num_atom
      newidx = idx + length(supp_list{k}) - 1;
      s_list(k) = fA(k)*pnorm(x(idx:newidx));
      s = s + s_list(k);
      idx = newidx+1;
    end
    
    if cg_par.verbose > 1
      cprintf('blue', 'iter = %d, f = %g, f_ori = %g, time = %g\n', ...
                    i, f, f_ori, sum(rtime.tm_all));
    end

    if sum(rtime.tm_all) > exp_par.max_time
      break;
    end
  end
  
  if cg_par.verbose > 0
    % Print summary of GCG
    ;
  end
    
  rtime.tm_all = rtime.tm_all(1:i);
  rtime.tm_loss = rtime.tm_loss(1:i);
  rtime.tm_po = rtime.tm_po(1:i);
  rtime.tm_conic = rtime.tm_conic(1:i);
  rtime.tm_local = rtime.tm_local(1:i);
  res.flist = res.flist(1:i);
  res.flist_ori = res.flist_ori(1:i);     
  res.rtime = rtime;
  res.iter = i;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % Convert a set of atoms into an original vector w
  function w = xlist_2_w(x)
    w = zeros(n, 1);
    idx = 1;
    for j = 1 : num_atom
      supp = supp_list{j};
      newidx = idx + length(supp) - 1;
      w(supp) = w(supp) + x(idx:newidx);
      idx = newidx + 1;
    end
  end
  
  % Objective of 2-D conic search
  function [f, g] = obj_2d_cone(ab)
    a = ab(1);  b = ab(2);
    cur_w = a*w;
    supp = supp_list{num_atom};
    cur_w(supp) = cur_w(supp) + b*v;
    [f, g] = loss(cur_w);
    f = f + lambda*(a*s+b);
    g = [g'*w+lambda*s; v'*g(supp)+lambda];
  end

  function [f, g] = obj_total_cone(weights)
    
    g = zeros(num_atom, 1);
    cur_w = zeros(n, 1);
    idx = 1;
    total_weight = 0;
    for j = 1 : num_atom-1
      supp = supp_list{j};
      newidx = idx + length(supp) - 1;
      cur_w(supp) = cur_w(supp) + weights(j)*x(idx:newidx);
      total_weight = total_weight + weights(j)*s_list(j);
      idx = newidx + 1;
    end
    
    total_weight = total_weight + weights(end);
    supp = supp_list{num_atom};
    cur_w(supp) = cur_w(supp) + weights(end)*v;
    
    [f1, g1] = loss(cur_w);
    f = f1 + lambda*total_weight;
    
    idx = 1;
    for j = 1 : num_atom-1
      supp = supp_list{j};
      newidx = idx + length(supp) - 1;
      g(j) = g1(supp)'*x(idx:newidx) + lambda*s_list(j);
      idx = newidx + 1;
    end
    supp = supp_list{num_atom};
    g(num_atom) = g1(supp)'*v + lambda;
  end


  % Objective of local search
  function [f, g] = obj_local(x)
    [f1, g1] = loss(xlist_2_w(x));
    
    f = 0;    g = zeros(length(x), 1);    idx = 1;
    for j = 1 : num_atom
      supp = supp_list{j};
      newidx = idx+length(supp)-1;
      tempx = x(idx:newidx);
      [f2, g2] = pnorm(tempx);
      f = f + fA(j)*f2;
      g(idx:newidx) = lambda*fA(j)*g2 + g1(supp);
      idx = newidx+1;
    end
    f = lambda*f + f1;
  end

  function [f, g] = pnorm(x)
    p = exp_par.p;
    
    if isinf(p) 
      % p = \infty, q = 1 (p is the ptil in the paper, q is the p in the paper).
      if cg_par.sigma < 0  % no smoothing
        [f, indmax] = max(abs(x));
        g = zeros(length(x),1); 
        g(indmax) = sign(x(indmax)) / length(indmax);
      else % use smoothing
        sigma = polar_par.sigma;
        y = abs(x);
        maxy = max(y);
        yexp = exp((y-maxy)/sigma);
        sumexp = sum(yexp);
        f = sigma*log(sumexp) + maxy;

        if nargout > 1
          g = yexp/sumexp .* sign(x);
        end
      end
    else  % never use smoothing for other norms
      y = abs(x);
      expx = y.^(p-1);
      sumexpx = sum(expx.*y);
      f = sumexpx^(1/p);
      if nargout > 1
        if f > 1e-6
          g = power(sumexpx, 1/p - 1)*(expx .* sign(x));
        else
          g = zeros(length(x),1);
        end
      end
    end
  end

end
